package org.eclipse.smila.sample.pipelet;

import org.apache.axis2.transport.http.HTTPConstants;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.smila.blackboard.Blackboard;
import org.eclipse.smila.datamodel.AnyMap;
import org.eclipse.smila.datamodel.Value;
import org.eclipse.smila.processing.Pipelet;
import org.eclipse.smila.processing.ProcessingException;
import org.eclipse.smila.processing.parameters.ParameterAccessor;
import org.eclipse.smila.processing.util.ProcessingConstants;
import org.eclipse.smila.processing.util.ResultCollector;

import com.empolis.smila.sample.helloworld.HelloWorldStub;

import demo.hw.server.SayHi;
import demo.hw.server.SayHiDocument;
import demo.hw.server.SayHiResponse;
import demo.hw.server.SayHiResponseDocument;

public class HelloWorldPipelet implements Pipelet {

  private final String PROP_IN_ATT_NAME = "IN_ATT_NAME";

  private final String PROP_OUT_ATT_NAME = "OUT_ATT_NAME";

  private AnyMap _config;

  private final Log _log = LogFactory.getLog(HelloWorldPipelet.class);

  public HelloWorldPipelet() {
  }

  /** {@inheritDoc} */
  @Override
  public void configure(final AnyMap configuration) throws ProcessingException {
    _config = configuration;
  }

  /** {@inheritDoc} */
  @Override
  public String[] process(final Blackboard blackboard, final String[] recordIds) throws ProcessingException {
    final ParameterAccessor paramAccessor = new ParameterAccessor(blackboard, _config);
    final ResultCollector resultCollector =
      new ResultCollector(paramAccessor, _log, ProcessingConstants.DROP_ON_ERROR_DEFAULT);
    for (final String id : recordIds) {
      try {
        paramAccessor.setCurrentRecord(id);
        final String inAttName = paramAccessor.getRequiredParameter(PROP_IN_ATT_NAME);
        if (inAttName.trim().length() == 0) {
          throw new ProcessingException("Property " + PROP_IN_ATT_NAME + " must not be an empty String");
        }
        final String outAttName = paramAccessor.getRequiredParameter(PROP_OUT_ATT_NAME);
        if (outAttName.trim().length() == 0) {
          throw new ProcessingException("Property " + PROP_OUT_ATT_NAME + " must not be an empty String");
        }

        String inputValue = "";
        if (blackboard.getMetadata(id).containsKey(inAttName)) {
          inputValue = blackboard.getMetadata(id).getStringValue(inAttName);
        }

        final HelloWorldStub ws =
          new HelloWorldStub("http://localhost:8081/axis2/services/HelloWorld.HelloWorldImplPort");
        ws._getServiceClient().getOptions().setProperty(HTTPConstants.CHUNKED, Boolean.FALSE);
        final SayHiDocument sayHiDoc = SayHiDocument.Factory.newInstance();
        final SayHi sayHi = sayHiDoc.addNewSayHi();
        sayHi.setArg0(inputValue);
        final SayHiResponseDocument respDoc = ws.sayHi(sayHiDoc);
        final SayHiResponse response = respDoc.getSayHiResponse();
        final String outputValue = response.getReturn();

        final Value outLiteral = blackboard.getDataFactory().createStringValue(outputValue);
        blackboard.getMetadata(id).put(outAttName, outLiteral);

        resultCollector.addResult(id);
      } catch (final Exception e) {
        resultCollector.addFailedResult(id, e);
      }
    }
    return resultCollector.getResultIds();
  }
}
